//#define _GNU_SOURCE

#include <stdio.h>
#include <string.h>
#include <time.h>

#include "ifbeam_c.h"

/*
 * Note: This code uses wda module. In order to compile it needs files wda.h and wda.o to be present.
 */

typedef struct {
    unsigned long size,resident,share,text,lib,data,dt;
} statm_t;

# if defined(__linux__)

void read_off_memory_status(statm_t *result)
{
    unsigned long dummy;
    const char* statm_path = "/proc/self/statm";

    FILE *f = fopen(statm_path,"r");
    if (!f) {
        perror(statm_path);
        abort();
    }
    if (7 != fscanf(f,"%ld %ld %ld %ld %ld %ld %ld",
        &result->size,&result->resident,&result->share,&result->text,&result->lib,&result->data,&result->dt))
    {
        perror(statm_path);
        abort();
    }
    fclose(f);
    fprintf(stderr, "********* total=%ldkB, resident=%ldkB, shared=%ldkB, text=%ldkB, (data+stack)=%ldkB *********\n\n",
        result->size*4, result->resident*4, result->share*4, result->text*4, result->data*4);
}

# define READ_OFF_MEMORY_STATUS(a) read_off_memory_status(a)

# else

# define READ_OFF_MEMORY_STATUS(a)

# endif


int main(void)
{
    int i, j, k;
    int err = -1234;
    statm_t stat;

//    const char *url = "http://dbdata0vm.fnal.gov:8099/ifbeam/data";
    // const char *url = "http://ifbcollectorgpvm04.fnal.gov:8088/ifbeam/data";
    const char *url = "http://ifbcollectorgpvm04.fnal.gov:9090/ifbeam/data";


    Dataset ds;
    Tuple tu;
    Measurement me;

    int len;
    int error;
    char ss[81920];
    double dd[8192];

    setUserAgent("Test_ifbeam_1");                    // Set User-Agent header for HTTP request

    time_t t0 = time(NULL);
//    ds = getBundleForInterval(url, "BNBShortTerm", t0-305, t0-300, &error);     // Get the data for bundle
//    ds = getBundleForInterval(url, "Weather", t0-500, t0-300, &error);
    ds = getBundleForInterval(url, "NuMI_Physics_A9", t0-320, t0-300, &error);
//    ds = getBundleForInterval(url, "NuMI_Physics_1Hz", t0-320, t0-300, &error);


    if (error) {                                            // Check for curl library errors
        fprintf(stderr, "main(): error code=%d(0x%x)\n", error, error);    perror("error message");
    }
    if (getHTTPstatus(ds) != 200) {                         // Check for HTTP error
        fprintf(stderr, "main(): HTTP code=%ld, message: '%s'\n", getHTTPstatus(ds), getHTTPmessage(ds));
    }


    fprintf(stderr, "#######Bundle for interval...\n");
    fprintf(stderr, "\nntuples=%d\n\n", getNtuples(ds));    // Get the number of rows in the dataset
# if 1
//    for (i = 0; i < getNtuples(ds); i++) {
    for (i = 0; i < 30; i++) {
        tu = getTuple(ds, i);                                     // Returns NULL if out of range
        len = getStringValue(tu, 0, ss, sizeof (ss), &err);       // Returns string length
        fprintf(stderr, "[%d]: len=%d, s0='%s'", i, len, ss);     // Print the results
        len = getStringValue(tu, 1, ss, sizeof (ss), &err);       // Returns string length
        fprintf(stderr, " s1='%s'\n", ss);                        // Print the results
    }
# endif
//==============================================

    fprintf(stderr, "\n#######The first tuple...\n");
//  tu = getTuple(ds, 1);                               // Returns NULL if out of range
    tu = getFirstTuple(ds);                             // Get the very first row - contains the column names
    // If we get the names print them
    if (tu != NULL) {                                               // If everything is OK
        int nc = getNfields(tu);                                    // Get the number of columns in this row
        fprintf(stderr, "ncols=%d\n", nc);

        for (i = 0; i < nc; i++) {                                  // Loop to get column data as a string
            len = getStringValue(tu, i, ss, sizeof (ss), &err);     // Returns string length
            fprintf(stderr, "[%d]: len=%d, s='%s'\n", i, len, ss);    // Print the results
        }
        fprintf(stderr, "err=%d\n", err);
        fprintf(stderr, "e=%s\n\n", strerror(err));                 // Was it OK?
    }



//==============================================

    fprintf(stderr, "#######The next tuple...\n");
    tu = getNextTuple(ds);                              // Get the next row, it contains the first data value in the dataset

    if (tu != NULL) {                                               // If everything is OK
        int nc = getNfields(tu);                                    // Get the number of columns in this row
        fprintf(stderr, "ncols=%d\n", nc);

        for (i = 0; i < nc; i++) {                                  // Loop to get column data as a string
            len = getStringValue(tu, i, ss, sizeof (ss), &err);     // Returns string length
            fprintf(stderr, "[%d]: len=%d, s='%s'\n", i, len, ss);    // Print the results
        }
        // Get double
        fprintf(stderr, "[3]: v=%f\n", getDoubleValue(tu, 3, &err));// Now extract the double value from the 3-rd column
        fprintf(stderr, "e=%s\n\n", strerror(err));                 // Was it OK?
    } else {
        fprintf(stderr, "No such tuple"); perror("...");
    }



//==============================================

    fprintf(stderr, "#######The 6-th tuple, should be array...\n");
    tu = getTuple(ds, 6);                                           // Get the row with double array

    if (tu != NULL) {                                               // If everything is OK
        int nc = getNfields(tu);                                    // Get the number of columns in this row
        for (i = 0; i < nc; i++) {                                  // Loop to get column data as a string
            len = getStringValue(tu, i, ss, sizeof (ss), &err);     // Returns string length
            fprintf(stderr, "[%d]: len=%d, s='%.25s'\n", i, len, ss); // Print the results
        }
        fprintf(stderr, "e=%s\n\n", strerror(err));                 // Was it OK?

        len = getStringValue(tu, 1, ss, sizeof (ss), &err);
        fprintf(stderr, "name='%s'\n", ss);
        fprintf(stderr, "ncols=%d\n", nc);

        // Get the first double
        // Now extract the double value from the 3-rd column
        fprintf(stderr, "[3]: fval=%f\n", getDoubleValue(tu, 3, &err));
        fprintf(stderr, "e=%s\n\n", strerror(err));                 // Was it OK?

        // Get double array
        len = getDoubleArray(tu, 3, dd, 4096, &err);                // Now extract the double array from the 3-rd column
        fprintf(stderr, "Array len=%d\n", len);                     // Print the length

        for (i = 0; i < len; i++) {                                 // Print the results
            fprintf(stderr, "%f, ", dd[i]);
        }
        fprintf(stderr, "\n");

        fprintf(stderr, "e=%s\n\n", strerror(err));                 // Was it OK?
    } else {
        fprintf(stderr, "No such tuple"); perror("...");
    }



//==============================================
# if 1
    fprintf(stderr, "#######The first Measurement...\n");
    me = getMeasurement(ds, 1);

    if (me != NULL) {
        fprintf(stderr, "clock=%ld\n", me->clock);
        fprintf(stderr, "device='%s'\n", me->device);
        fprintf(stderr, "units='%s'\n", me->units);
        fprintf(stderr, "vector_size=%zu\n", me->vector_size);
        if (me->vector_size > 0) {
            len = me->vector_size;
            fprintf(stderr, "********* vec=%p\n", me->vector);      // debug
            for (i = 0; i < len; i++) {                             // Print the results
                fprintf(stderr, "%f, ", (me->vector)[i]);
            }
            fprintf(stderr, "\n\n");
        } else if (me->vector_size == 0) {
            fprintf(stderr, "value=%f\n\n", me->value);
        } else {
            fprintf(stderr, "Something is wrong with the measurement\n");
            perror("Something is wrong with the measurement");
        }
    } else {
        fprintf(stderr, "No such measurement\n");
        perror("No such measurement");
    }

# endif

    //
    READ_OFF_MEMORY_STATUS(&stat);
    //
    fprintf(stderr, "#######Release dataset...\n");
    releaseDataset(ds);                                             // Release dataset to prevent memory leak!
    //
    READ_OFF_MEMORY_STATUS(&stat);
    //



# if 1
//==============================================
    fprintf(stderr, "\n\n\n#######Event for interval...\n");
//  ds = getEventVarForInterval(url, "e,8f", "E:HMGPR", 1423851431.000, 1423851436.000, &error);     // Get the data for the variable in event
    ds = getEventVarForInterval(url, "e,8f", "E:HMGPR", t0-305, t0-300, &error);     // Get the data for the variable in event
//  ds = getEventVarForTime    (url, "e,8f", "E:HMGPR", t0-305,         &error);     // Get the data for the variable in event
    fprintf(stderr, "\nntuples=%d\n\n", getNtuples(ds));    // Get the number of rows in the dataset

    tu = getFirstTuple(ds);                                 // Get the very first row - contains the column names
    tu = getNextTuple(ds);                                  // Get the first data row
    // If we get the names print them
    if (tu != NULL) {                                               // If everything is OK
        int nc = getNfields(tu);                                    // Get the number of columns in this row
        fprintf(stderr, "ncols=%d\n", nc);

        for (i = 0; i < nc; i++) {                                  // Loop to get column data as a string
            len = getStringValue(tu, i, ss, sizeof (ss), &err);     // Returns string length
            fprintf(stderr, "[%d]: len=%d, s='%s'\n", i, len, ss);  // Print the results
        }
        fprintf(stderr, "e=%s\n\n", strerror(err));                 // Was it OK?
    }
        // Get double array
        len = getDoubleArray(tu, 4, dd, 4096, &err);                // Now extract the double array from the 3-rd column
        fprintf(stderr, "Array len=%d\n", len);                     // Print the length

        for (i = 0; i < len; i++) {                                 // Print the results
            fprintf(stderr, "%f, ", dd[i]);
        }
        fprintf(stderr, "\n\n");
# if 1
    fprintf(stderr, "#######The first Measurement...\n");
    me = getMeasurement(ds, 1);

    if (me != NULL) {
        fprintf(stderr, "clock=%ld\n", me->clock);
        fprintf(stderr, "device='%s'\n", me->device);
        fprintf(stderr, "units='%s'\n", me->units);
        fprintf(stderr, "vector_size=%zu\n", me->vector_size);
        if (me->vector_size > 0) {
            len = me->vector_size;
            fprintf(stderr, "********* vec=%p\n", me->vector);      // debug
            for (i = 0; i < len; i++) {                             // Print the results
                fprintf(stderr, "%f, ", me->vector[i]);
            }
            fprintf(stderr, "\n\n");
        } else if (me->vector_size == 0) {
            fprintf(stderr, "value=%f\n\n", me->value);
        } else {
            fprintf(stderr, "Something is wrong with the measurement\n");
            perror("Something is wrong with the measurement");
        }
    } else {
        fprintf(stderr, "No such measurement\n");
        perror("No such measurement");
    }
# endif


//==============================================

    tu = getNextTuple(ds);                              // Get the next row, it contains the first data value in the dataset

    if (tu != NULL) {                                               // If everything is OK
        int nc = getNfields(tu);                                    // Get the number of columns in this row
        fprintf(stderr, "ncols=%d\n", nc);

        for (i = 0; i < nc; i++) {                                  // Loop to get column data as a string
            len = getStringValue(tu, i, ss, sizeof (ss), &err);     // Returns string length
            fprintf(stderr, "[%d]: len=%d, s='%s'\n", i, len, ss);  // Print the results
        }
        // Get double
        fprintf(stderr, "[4]: parsed v=%f\n", getDoubleValue(tu, 4, &err));// Now extract the double value from the 4-th column
        fprintf(stderr, "e=%s\n\n", strerror(err));                 // Was it OK?
    } else {
        fprintf(stderr, "No such tuple"); perror("...");
    }



    //
    READ_OFF_MEMORY_STATUS(&stat);
    //
    fprintf(stderr, "#######Release dataset...\n");
    releaseDataset(ds);                                             // Release dataset to prevent memory leak!
    //
    READ_OFF_MEMORY_STATUS(&stat);
    //
# endif

    return 0;
}


